@@ -21,6 +21,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
@@ -376,10 +380,14 @@ static void
 notify_progress (ECalBackendMAPI *cbmapi, guint64 index, guint64 total)
 {
 	guint percent = ((float)index/total) * 100 ;
+	gchar *progress_string;
+
 	if (percent > 100)
 		percent = 99; 
 
-	gchar *progress_string = g_strdup_printf (_("Loading %s items"), get_element_type (e_cal_backend_get_kind (E_CAL_BACKEND (cbmapi)))); 
+	/* To translators: This message is displayed on the status bar when calendar/tasks/memo items are being fetched from the server. */
+	progress_string = g_strdup_printf (_("Loading items in folder %s"), 
+				e_source_get_property (e_cal_backend_get_source (E_CAL_BACKEND (cbmapi)), "name")); 
 
 	e_cal_backend_notify_view_progress (E_CAL_BACKEND (cbmapi), progress_string, percent);
 
@@ -567,8 +575,8 @@ get_deltas (gpointer handle)
 						NULL, 0, NULL, NULL, 
 						mapi_cal_get_changes_cb, cbmapi, 
 						MAPI_OPTIONS_FETCH_ALL)) {
-			/* FIXME: better string please... */
-			e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server. Removing the cache might help."));
+			/* FIXME: String : We need to restart evolution-data-server */
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server."));
 //			e_file_cache_thaw_changes (E_FILE_CACHE (priv->cache));
 			g_static_mutex_unlock (&updating);
 			return FALSE;
@@ -578,8 +586,8 @@ get_deltas (gpointer handle)
 						exchange_mapi_cal_util_build_name_id, GINT_TO_POINTER(kind), 
 						mapi_cal_get_changes_cb, cbmapi, 
 						MAPI_OPTIONS_FETCH_ALL)) {
-		/* FIXME: better string please... */
-		e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server. Removing the cache might help."));
+		/* FIXME: String : We need to restart evolution-data-server */
+		e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server."));
 //		e_file_cache_thaw_changes (E_FILE_CACHE (priv->cache));
 		g_static_mutex_unlock (&updating);
 		return FALSE;
@@ -604,8 +612,8 @@ get_deltas (gpointer handle)
 						NULL, NULL, 
 						handle_deleted_items_cb, cbmapi, 
 						0)) {
-		/* FIXME: better string please... */
-		e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server. Removing the cache might help."));
+		/* FIXME: String : We need to restart evolution-data-server */
+		e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server."));
 		priv->cache_keys = NULL;
 		g_static_mutex_unlock (&updating);
 		return FALSE;
@@ -1294,6 +1302,7 @@ e_cal_backend_mapi_create_object (ECalBackendSync *backend, EDataCal *cal, char
 	GSList *streams = NULL;
 	struct cbdata cbdata;
 	struct Binary_r globalid;
+	struct icaltimetype current;
 
 	cbmapi = E_CAL_BACKEND_MAPI (backend);
 	priv = cbmapi->priv;
@@ -1318,6 +1327,10 @@ e_cal_backend_mapi_create_object (ECalBackendSync *backend, EDataCal *cal, char
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
 
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+ 	e_cal_component_set_created (comp, &current);
+ 	e_cal_component_set_last_modified (comp, &current);
+
 	/* FIXME: [WIP] Add support for recurrences */
 	if (e_cal_component_has_recurrences (comp)) {
 		GByteArray *ba = exchange_mapi_cal_util_rrule_to_bin (comp, NULL); 
@@ -1329,6 +1342,9 @@ e_cal_backend_mapi_create_object (ECalBackendSync *backend, EDataCal *cal, char
 			if (tag_array) {
 				stream->proptag = tag_array->aulPropTag[0];
 				streams = g_slist_append (streams, stream); 
+
+				g_free (tag_array->aulPropTag); 
+				g_free (tag_array);
 			}
 		}
 	}
@@ -1477,7 +1493,8 @@ e_cal_backend_mapi_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	GSList *attachments = NULL;
 	struct cbdata cbdata;
 	gboolean no_increment = FALSE;
-	icalproperty *prop; 
+	icalproperty *prop;
+	struct icaltimetype current;
 
 	*old_object = *new_object = NULL;
 	cbmapi = E_CAL_BACKEND_MAPI (backend);
@@ -1513,6 +1530,9 @@ e_cal_backend_mapi_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
 
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+ 	e_cal_component_set_last_modified (comp, &current);
+
 	/* FIXME: [WIP] Add support for recurrences */
 	if (e_cal_component_has_recurrences (comp)) {
 		GByteArray *ba = exchange_mapi_cal_util_rrule_to_bin (comp, NULL); 
@@ -1659,7 +1679,7 @@ e_cal_backend_mapi_remove_object (ECalBackendSync *backend, EDataCal *cal,
 			/*remove a single instance of a recurring event and modify */
 			time_rid = icaltime_from_string (rid);
 			e_cal_util_remove_instances (icalcomp, time_rid, mod);
-			new_calobj  = (char *) icalcomponent_as_ical_string (icalcomp);
+			new_calobj  = (char *) icalcomponent_as_ical_string_r (icalcomp);
 			status = e_cal_backend_mapi_modify_object (backend, cal, new_calobj, CALOBJ_MOD_ALL, &obj, &new_object);
 			if (status == GNOME_Evolution_Calendar_Success) {
 				*old_object = obj;
@@ -1752,13 +1772,26 @@ e_cal_backend_mapi_send_objects (ECalBackendSync *backend, EDataCal *cal, const
 			mapi_id_t mid = 0;
 			GSList *recipients = NULL;
 			GSList *attachments = NULL;
+			GSList *streams = NULL;
 
 			e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (subcomp));
 
 			/* FIXME: Add support for recurrences */
 			if (e_cal_component_has_recurrences (comp)) {
-				g_object_unref (comp);
-				return GNOME_Evolution_Calendar_OtherError;
+				GByteArray *ba = exchange_mapi_cal_util_rrule_to_bin (comp, NULL); 
+				if (ba) {
+					struct SPropTagArray *tag_array; 
+					ExchangeMAPIStream *stream = g_new0 (ExchangeMAPIStream, 1); 
+					stream->value = ba; 
+					tag_array = exchange_mapi_util_resolve_named_prop (priv->olFolder, priv->fid, 0x8216, PSETID_Appointment); 
+					if (tag_array) {
+						stream->proptag = tag_array->aulPropTag[0];
+						streams = g_slist_append (streams, stream); 
+
+						g_free (tag_array->aulPropTag); 
+						g_free (tag_array);
+					}
+				}
 			}
 
 			if (e_cal_component_has_attachments (comp))
@@ -1806,7 +1839,7 @@ e_cal_backend_mapi_send_objects (ECalBackendSync *backend, EDataCal *cal, const
 			mid = exchange_mapi_create_item (olFolderOutbox, 0, 
 							exchange_mapi_cal_util_build_name_id, GINT_TO_POINTER(kind), 
 							exchange_mapi_cal_util_build_props, &cbdata, 
-							recipients, attachments, NULL, 0);
+							recipients, attachments, streams, 0);
 			g_free (cbdata.props);
 			if (!mid) {
 				g_object_unref (comp);
@@ -1891,7 +1924,7 @@ e_cal_backend_mapi_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 					icalcomponent_set_method (resp_comp, ICAL_METHOD_RESPONSE);
 					icalcomponent_add_component (resp_comp, 
 						icalcomponent_new_clone(e_cal_component_get_icalcomponent(comp)));
-					comp_str = icalcomponent_as_ical_string (resp_comp);
+					comp_str = icalcomponent_as_ical_string_r (resp_comp);
 					status = e_cal_backend_mapi_send_objects (backend, cal, comp_str, &users, &new_object);
 					g_free (comp_str);
 					g_free (new_object);
@@ -1957,7 +1990,7 @@ e_cal_backend_mapi_get_timezone (ECalBackendSync *backend, EDataCal *cal, const
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
 
-	*object = icalcomponent_as_ical_string (icalcomp);
+	*object = icalcomponent_as_ical_string_r (icalcomp);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -2210,6 +2243,7 @@ e_cal_backend_mapi_set_mode (ECalBackend *backend, CalMode mode)
 {
 	ECalBackendMAPI *cbmapi;
 	ECalBackendMAPIPrivate *priv;
+	gboolean re_open = FALSE;
 
 	cbmapi = E_CAL_BACKEND_MAPI (backend);
 	priv = cbmapi->priv;
@@ -2222,6 +2256,8 @@ e_cal_backend_mapi_set_mode (ECalBackend *backend, CalMode mode)
 
 	g_mutex_lock (priv->mutex);
 
+	re_open = priv->mode == CAL_MODE_LOCAL && mode == CAL_MODE_REMOTE;
+
 	priv->mode_changed = TRUE;
 	switch (mode) {
 		case CAL_MODE_REMOTE:
@@ -2230,7 +2266,7 @@ e_cal_backend_mapi_set_mode (ECalBackend *backend, CalMode mode)
 			e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
 					GNOME_Evolution_Calendar_MODE_REMOTE);
 			e_cal_backend_notify_readonly (backend, priv->read_only);
-			if (e_cal_backend_mapi_is_loaded (backend))
+			if (e_cal_backend_mapi_is_loaded (backend) && re_open)
 		              e_cal_backend_notify_auth_required(backend);
 			break;
 		case CAL_MODE_LOCAL:
